@@ -18,8 +18,9 @@ class AbstractHeaderAuthenticator(AuthBase):
 
     def get_auth_header(self) -> Mapping[str, Any]:
         """The header to set on outgoing HTTP requests"""
-
-        return {self.auth_header: self.token}
+        if self.auth_header:
+            return {self.auth_header: self.token}
+        return {}
 
     @property
     @abstractmethod
